#include <cvd/config.h>
#include <cvd/byte.h>


//Buffer types
//DiskBuffer2
//VideoFileBuffer (opt)
//v4l2buffer (opt)
//v4l1buffer (opt)
//v4lbuffer (opt)
//dvbuffer2 (opt)
//o2buffer (opt)
//os


#ifdef CVD_HAVE_V4L2_BUFFER
	#include <cvd/Linux/v4l2buffer.h>
	#include <cvd/Linux/v4lbuffer.h>
#endif

#ifdef CVD_HAVE_V4L1_BUFFER
	#include <cvd/Linux/v4l1buffer.h>
#endif

#ifdef CVD_HAVE_DVBUFFER
	#include <cvd/Linux/dvbuffer.h>
#endif

#ifdef CVD_HAVE_FFMPEG
	#include <cvd/videofilebuffer.h>
#endif

#include <cvd/diskbuffer2.h>

#include "vid_buffer.h"
#include <gvars3/instances.h>


using namespace CVD;
using namespace std;
using namespace GVars3;


struct GoneBad: public CVD::Exceptions::All
{
	GoneBad::GoneBad(string s)
	{
		what = "Can't instantiate " + s + " for the requested type.";
	}
};

////////////////////////////////////////////////////////////////////////////////
//
// DiskBuffer
// 

template<class C>  VideoBuffer<C>*  glob(string stub)
{
	gvar2_string	glob;
	gvar2_int		skip;
	GV2.Register(glob, stub+"disk_glob.glob", "/dev/null", false);
	GV2.Register(skip, stub+"disk_glob.skip", -1, true);

	DiskBuffer2<C>* p;
		
	if(*skip == -1)
		p = new DiskBuffer2<C>(globlist(*glob), 25);
	else
	{
		vector<string> files, matches;

		matches = globlist(*glob);

		for(int i=0; i < (int)matches.size(); i++)
			if(i >= *skip)
				files.push_back(matches[i]);

		p = new DiskBuffer2<C>(files, 25);
	}

	p->on_end_of_buffer(VideoBufferFlags::UnsetPending);

	return p;
}

////////////////////////////////////////////////////////////////////////////////
//
// VideoFileBuffer
// 

#ifdef CVD_HAVE_FFMPEG
template<class C>  VideoBuffer<C>* avi(string stub)
{
	string file = GV3::get<string>(stub + "avi.file");
	return new VideoFileBuffer<byte>(file);
}
#endif


////////////////////////////////////////////////////////////////////////////////
//
// DVBuffer
//

template<class C>  VideoBuffer<C>* dv(string stub)
{
	return 0;
}

#ifdef CVD_HAVE_DVBUFFER
template<> VideoBuffer<Rgb<byte> >* dv<Rgb<byte> >(string stub)
{
	gvar2_int	camera;
	GV2.Register(camera, stub+"dvbuffer.camera_number", 0, true);
	DVBuffer2<Rgb<byte> > *b=new DVBuffer2<Rgb<byte> >(*camera, 3);
	return b;
}

template<> VideoBuffer<byte>* dv<byte>(string stub)
{
	gvar2_int	camera;
	GV2.Register(camera, stub+"dvbuffer.camera_number", 0, true);
	DVBuffer2<byte> *b=new DVBuffer2<byte>(*camera, 3);
	return b;
}
#endif

////////////////////////////////////////////////////////////////////////////////
//
// V4L2Buffer
//

#ifdef CVD_HAVE_V4L2_BUFFER

template<class C> v4l2(string)
{
	return 0;
}

template<>  v4l2<byte>(string stub)
{
	gvar2_string	vid_file;
	GV2.Register(vid_file, stub+"v4l2buffer.video_device", "/dev/video0", true);

	gvar2_int		use_fields;
	GV2.Register(use_fields, stub+"v4l2buffer.use_fields", 0, true);
	
	return new V4L2Buffer((*vid_file).c_str(), *use_fields, V4L2BBMsleep);
}
#endif


template<class C>  VideoBuffer<C>* vid_buffer<C>::get(string stub)
{
	if(stub != "")
		stub += ".";
	
	gvar2_string cam_type;

	GV2.Register(cam_type, stub+"buffer_type", "v4l2", false);
	VideoBuffer<C>* r;
	if(*cam_type == "disk_glob")
		r = glob<C>(stub);
	#ifdef CVD_HAVE_V4L2_BUFFER
	else  if(*cam_type == "v4l2")
		r = v4l2<C>(stub);
	#endif
	#ifdef CVD_HAVE_FFMPEG
	else if(*cam_type == "avi")
		r = avi<C>(stub);
	#endif
	#ifdef CVD_HAVE_DVBUFFER
	else if(*cam_type == "dv")
		r = dv<C>(stub);
	#endif
	else
	{
		cerr << "Error: buffer type: " << *cam_type << " does not exist on this system." << endl;
		exit(1);
	}

	if(!r)
		throw GoneBad(*cam_type);
	
	return r;
}
	



void _inst_func()
{
	vid_buffer<byte>::get("");
}


